/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR235x Demo: High Side current sensing using SAC0 and ADC window 
//                     comparator
//
//  Description: Current is measured using an external shunt resistor connected
//  to the amplifier. The output is connected to an ADC which uses window 
//  comparator mode to detect high/low thresholds. 
//  Configure SAC0 in General Purpose (GP) mode with external resistors. 
//  The ADC is triggered by timer TB1.1 every ~0.05 seconds (SAMPLING FREQ) 
//  without CPU intervention. 
//  When the signal drops below LO_TRESHOLD, LED1 turns on. 
//  When the signal exceeds HI_TRESHOLD, LED2 turns on. When the signal is 
//  between the 2 thresholds, both LEDs are off. 
//  ACLK = XT1 = 32768 Hz, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR235x
//             -------------------
//         /|\|          P1.2/OA0-|----|---------R4=1.01Kohm----
//          | |                   | R5=50.5Kohm                 |
//          | |       P1.1/OA0O/A1|----|------>Vout/ADC      R1=200mOhm (shunt)
//          --|RST                |                             |
//            |          P1.3/OA0+|<---|---------R2=1.01KOhm----|
//            |                   |  R3=50.5KOhm
//            |                   |    |
//            |                   |   GND
//            |                   |
//            |                   |
//            |               P1.0|---> LED1 (turns on below ADCLO)
//            |               P6.6|---> LED2 (turns on above ADCHI)
//            |                   |
//
//   Luis Reynoso
//   Texas Instruments Inc.
//   Nov 2019
//   Built with IAR Embedded Workbench v7.12.1 & Code Composer Studio v9.0.1 
//******************************************************************************
#include <msp430.h>

#define HI_THRESHOLD    (2482)                  // High treshold = 200mA
                                                // Assume V(R+) = AVCC = 3.3 V
                                                // 200mA x 200mOhm = 40mV
                                                // 40mV x 50V/V Gain = 2V
                                                // 2V*(2^12-bit)/3.3V = 2482

#define LO_THRESHOLD    (620)                   // Low treshold = 50mA
                                                // Assume V(R+) = AVCC = 3.3 V
                                                // 50mA x 200mOhm = 10mV
                                                // 10mV x 50V/V Gain = 500mV
                                                // 0.5V*(2^10-bit)/3.3V = 620
#define SAMPLING_FREQ   (20)                    // Sample at 20Hz (0.05s)
#define SAMPLING_CYCLES (327682/SAMPLING_FREQ)  // Using ACLK = 32.768kHz 
                                         
                                         
int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;                 // Stop watch dog timer

    // Configure LED1 (P1.0) and LED2 (P6.6)
    P1DIR |= BIT0;                       // Set P1.0 direction as output
    P1OUT &= ~BIT0;                      // Set P1.0 output low
    P6DIR |= BIT6;                       // Set P6.6 direction as output
    P6OUT &= ~BIT6;                      // Set P6.6 output low
    
    // Disable GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Configure ADC to sample SAC0 output (A1) using window comparator
    ADCCTL0 &= ~ADCENC;                       // Disable ADC
    ADCCTL0 = ADCSHT_2 | ADCON;            // 16 ADCCLK cycles; Enable ADC core
    ADCCTL1 = ADCSHP | ADCSHS_2 |          // Pulse sample mode; TB1.1 trigger
              ADCCONSEQ_2 |                // Repeat-single-channel mode
              ADCSSEL_0 | ADCDIV_0;        // ADCCLK = MODCLK, divide by 1

    ADCCTL2 = ADCRES_2;                    // 12-bit conversion results
    ADCMCTL0 = ADCINCH_1 | ADCSREF_0;      // A1 ch; V(R+) = AVCC, V(R-) = AVSS
    ADCHI = HI_THRESHOLD;                  // Set comparator high threshold
    ADCLO = LO_THRESHOLD;                  // Set comparator low threshold
    ADCIE |= ADCHIIE | ADCLOIE | ADCINIE;  // Enable ADCHI, ADCLO, ADCIN
                                           // interrupts

    // Configure SAC0 in GP mode
    P1SEL0 |= BIT1 | BIT2 | BIT3;        // Select OA0O (P1.1), OA0- (P1.2)
    P1SEL1 |= BIT1 | BIT2 | BIT3;        // and OA0+ (P1.3) functions

    SAC0OA &= ~(NSEL_0 | PSEL_0);        // Select external sources for SAC0
                                         // negative and positive inputs

    SAC0OA |= NMUXEN | PMUXEN;           // Connect SAC0 negative and positive
                                         // input sources

    SAC0OA |= OAPM;                      // Select low speed and low-power mode
    SAC0OA |= SACEN | OAEN;              // Enable SAC0 and OA in normal mode    
    
    // Configure ADC timer trigger TB1.1
    TB1CCR0 = (SAMPLING_CYCLES-1);       // PWM period
    TB1CCR1 = (SAMPLING_CYCLES/2);       // 50% duty cycle; TB1.1
    TB1CCTL1 = OUTMOD_3;                 // TB1CCR1 set/reset mode
    TB1CTL = TBSSEL_1 | MC_1 | TBCLR;    // ACLK clock source, up mode, clear
    ADCCTL0 |= ADCENC;                   // Enable ADC conversions
  
    
    while(1)
    {
        __bis_SR_register(LPM0_bits | GIE);  // Enter LPM0, enable interrupts
        __no_operation();                    // For debugger
    }

}

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV,ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:           // ADCHI; ADC channel > HI_THRESHOLD
            P1OUT &= ~BIT0;            // Turn off LED1
            P6OUT |= BIT6;             // Turn on LED2
            break;
        case ADCIV_ADCLOIFG:           // ADCLO; ADC channel < LO_THRESHOLD
            P6OUT &= ~BIT6;            // Turn on LED1
            P1OUT |= BIT0;             // Turn off LED2
            break;
        case ADCIV_ADCINIFG:           // ADCIN; LO_THRESHOLD < ADC channel <
                                       // HI_THRESHOLD
            P1OUT &= ~BIT0;            // Turn off LED1
            P6OUT &= ~BIT6;            // Turn off LED2
            break;
        case ADCIV_ADCIFG:
            break;
        default:
            break;
    }
}
